/*
 when we treat the ship as a point, nose of the ship should be at the coordinates,
 this way we can give a missile that comesout from the front of the ship same coordinates,
 and after new location calculations missile should clear the ship.
*/

import java.util.*; // Linked List 
import java.awt.*; // Graphics


class Ship extends Sprite{
String shipName;
boolean thrusterOn = false;
boolean fireMissile = false;

Vector2d velocity;
Vector2d orientation;

// ACCELERATION value is relative to initial direction vector measured in Astro Units
static double G = 0.003; // G is a measure of proportionality of G force excerted by the star
static double ACCELERATION = 0.003; // how fast can we accelerate
static double TURN_VELOCITY = 0.3; // Radians
double angularVelocity = 0.0; // how fast can we turn?
static double SHIP_RATIO = 0.06; // how large is a ship compared to orientation unit vector
static int INVALID_VALUE = -1; // are we counting tics or tick value is invalid ( no explosion happening )
int explodedTics = INVALID_VALUE;
static double MAX_SPEED = 0.2;

	public void run(ListIterator sprites){ // update internal state of the object
		location.x = (location.x<0) ? Window.XOFF:location.x; // off the screen?
		location.x = (location.x>Window.XOFF) ? 0:location.x;
		location.y = (location.y<0) ? Window.YOFF:location.y;
		location.y = (location.y>Window.XOFF) ? 0:location.y;

		if(collision){ // collision with a missile, ship or star has occured
			if(explodedTics == INVALID_VALUE){
				sprites.add(new Explosion(location));
				explodedTics = SWar.tics; // remember when we exploded
			} else if(SWar.tics > explodedTics + Explosion.MAX_LIVE_CYCLES*2){
				init(shipName); // reinitialize ship position, explodedTics and collision flag
			}
			return;
		}

		double cosAV = Math.cos(angularVelocity); // turn by angularVelocity
		double sinAV = Math.sin(angularVelocity);
		double x = orientation.x;
		double y = orientation.y;
		orientation.x = x*cosAV - y*sinAV;
		orientation.y = x*sinAV + y*cosAV;

		if(thrusterOn){ // adjust velocity if the thruster is on
			Vector2d newVel = velocity.add(orientation.scale(ACCELERATION));
			if( velocity.scalarValue() < MAX_SPEED || newVel.scalarValue() < velocity.scalarValue() ){
				velocity = newVel; // top off the velocity
			}
		}

		// calculate displacement wrt the sun's gravity
		Vector2d star = new Vector2d(Star.STAR_X, Star.STAR_Y);
		Vector2d gravity = star.sub(location);
		double scalarg = gravity.scalarValue();
		gravity = gravity.scale( G/(scalarg*scalarg));

		location = location.add(gravity); // change ship position wrt gravity/velocity
		location = location.add(velocity);
		if(fireMissile){ // introduce a missile into the game
			Sprite miss = new Missile(velocity,orientation,location);
			sprites.add(miss); // add it to the list to be run
			fireMissile = false;
		}
	}


	public void go(boolean onOff){ // accelerate the ship
		thrusterOn = onOff;
	}
	public void left(boolean onOff){
		angularVelocity = onOff? TURN_VELOCITY : 0;
	}
	public void right(boolean onOff){
		angularVelocity = onOff? -TURN_VELOCITY:0;
	}
	public void fire(){
		fireMissile = true;
	}


	public void draw(Graphics g){ // let the object draw itself on the buffer
		if(collision){
			return; // do not draw the ship while explosion lasts
		}
		g.setColor(Color.white);
		Point pt = Window.toScreen(location); // convert location in AU to screen coordinates
		Vector2d shipEnd = location.add(orientation.scale(SHIP_RATIO));
		Point se = Window.toScreen(shipEnd);
		g.drawLine(pt.x, pt.y, se.x, se.y);
		g.fillOval(pt.x-9,pt.y-9,18,18);
		g.setColor(Color.black);
		g.fillOval(pt.x-5,pt.y-5,10,10);
	}

	Ship(String name){ init(name); } // we reuse init during run()

	public void init(String name){
		shipName = name; // could be used to load image from file or define shape
		double x = SWar.rand.nextFloat()*(Window.SCREEN_WIDTH/Window.PIXEL_PER_AU);
		double y = SWar.rand.nextFloat()*(Window.SCREEN_HEIGHT/Window.PIXEL_PER_AU);
		location = new Vector2d(x,y);
		velocity = new Vector2d(0,0);
		orientation = new Vector2d(1.0,0.0); // make it random?
		size = 9.0/Window.PIXEL_PER_AU; // we make an exception to magic # policy ;)
		explodedTics = INVALID_VALUE;
		collision = false;
	}

} // class Ship